@@ -42,6 +42,8 @@ defaultsCfg.set('USER_CONFIG_DEFAULTS', 'sbatch_qos', 'cp3')
 defaultsCfg.set('USER_CONFIG_DEFAULTS', 'sbatch_time', '0-04:00')
 defaultsCfg.set('USER_CONFIG_DEFAULTS', 'sbatch_memPerCPU', '2048')
 defaultsCfg.set('USER_CONFIG_DEFAULTS', 'scratchDir', '${LOCALSCRATCH}')
+defaultsCfg.set('USER_CONFIG_DEFAULTS', 'apptainer', 'false')
+defaultsCfg.set('USER_CONFIG_DEFAULTS', 'apptainerImage', '')
 defaultsCfg.add_section('USER_CONFIG_VALID_VALUES')
 defaultsCfg.set('USER_CONFIG_VALID_VALUES', 'sbatch_partition', 'Def,cp3')
 defaultsCfg.set('USER_CONFIG_VALID_VALUES', 'sbatch_qos', 'normal,cp3')
@@ -81,36 +83,42 @@ validConfigParams = collections.OrderedDict({
     'sbatch_output'               : {'mandatory': False, 'type': str,                        'default': '',                                                          'sbatch_option_long': 'output',      'sbatch_option_short': 'o'},
     'sbatch_error'                : {'mandatory': False, 'type': str,                        'default': '',                                                          'sbatch_option_long': 'error',       'sbatch_option_short': 'e'},
     'sbatch_additionalOptions'    : {'mandatory': False, 'type': list, 'element_type': str,  'default': []},
-    'scratchDir'                  : {'mandatory': False, 'type': str,                        'default': defaultsCfg.get('USER_CONFIG_DEFAULTS', 'scratchDir')},
-    'handleScratch'               : {'mandatory': False, 'type': bool,                       'default': False},
-    'environmentType'             : {'mandatory': False, 'type': str,                        'default': ''},
-    'cmsswDir'                    : {'mandatory': False, 'type': str,                        'default': ''},
+    'batchScriptsDir'             : {'mandatory': False, 'type': str,                        'default': '<config.sbatch_chdir>'},
+    'batchScriptsFilename'        : {'mandatory': False, 'type': str,                        'default': 'slurm_batch_script_{}_{}.sh'.format(timestamp, randomstr)},
     'inputSandboxContent'         : {'mandatory': False, 'type': list, 'element_type': str,  'default': []},
     'inputSandboxDir'             : {'mandatory': False, 'type': str,                        'default': '<config.sbatch_chdir>'},
     'inputSandboxFilename'        : {'mandatory': False, 'type': str,                        'default': 'input_sandbox_{}_{}.tar.gz'.format(timestamp, randomstr)},
-    'batchScriptsDir'             : {'mandatory': False, 'type': str,                        'default': '<config.sbatch_chdir>'},
-    'batchScriptsFilename'        : {'mandatory': False, 'type': str,                        'default': 'slurm_batch_script_{}_{}.sh'.format(timestamp, randomstr)},
-    'stageout'                    : {'mandatory': False, 'type': bool,                       'default': True},
-    'stageoutFiles'               : {'mandatory': False, 'type': list, 'element_type': str,  'default': []},
-    'stageoutDir'                 : {'mandatory': False, 'type': str,                        'default': '<config.sbatch_chdir>'},
+    'scratchDir'                  : {'mandatory': False, 'type': str,                        'default': defaultsCfg.get('USER_CONFIG_DEFAULTS', 'scratchDir')},
+    'handleScratch'               : {'mandatory': False, 'type': bool,                       'default': False},
     'writeLogsOnWN'               : {'mandatory': False, 'type': bool,                       'default': True},
     'separateStdoutStderrLogs'    : {'mandatory': False, 'type': bool,                       'default': False},
     'stdoutFilename'              : {'mandatory': False, 'type': str,                        'default': ''},
     'stderrFilename'              : {'mandatory': False, 'type': str,                        'default': ''},
-    'stageoutLogs'                : {'mandatory': False, 'type': bool,                       'default': True},
-    'stageoutLogsDir'             : {'mandatory': False, 'type': str,                        'default': '<config.stageoutDir>/logs'},
+    'environmentType'             : {'mandatory': False, 'type': str,                        'default': ''},
+    'cmsswDir'                    : {'mandatory': False, 'type': str,                        'default': ''},
     'useJobArray'                 : {'mandatory': False, 'type': bool,                       'default': True},
     'maxRunningJobs'              : {'mandatory': False, 'type': int,                        'default': None},
     'numJobs'                     : {'mandatory': False, 'type': int,                        'default': None},
+    'stageout'                    : {'mandatory': False, 'type': bool,                       'default': True},
+    'stageoutFiles'               : {'mandatory': False, 'type': list, 'element_type': str,  'default': []},
+    'stageoutDir'                 : {'mandatory': False, 'type': str,                        'default': '<config.sbatch_chdir>'},
+    'stageoutLogs'                : {'mandatory': False, 'type': bool,                       'default': True},
+    'stageoutLogsDir'             : {'mandatory': False, 'type': str,                        'default': '<config.stageoutDir>/logs'},
     'inputParamsNames'            : {'mandatory': False, 'type': list, 'element_type': str,  'default': []},
     'inputParams'                 : {'mandatory': False, 'type': list, 'element_type': list, 'default': []},
-    'doCheckpointing'             : {'mandatory': False, 'type': bool,                       'default': False},
-    'checkpointSignal'            : {'mandatory': False, 'type': str,                        'default': ""},
-    'checkpointSecsBeforeTimeout' : {'mandatory': False, 'type': int,                        'default': 0},
+    'putInputParamsInBatchScript' : {'mandatory': False, 'type': bool,                       'default': True},
+    'inputParamsFilename'         : {'mandatory': False, 'type': str,                        'default': 'input_params_{}_{}.sh'.format(timestamp, randomstr)},
+    'apptainer'                   : {'mandatory': False, 'type': bool,                       'default': defaultsCfg.getboolean('USER_CONFIG_DEFAULTS', 'apptainer')},
+    'apptainerImage'              : {'mandatory': False, 'type': str,                        'default': defaultsCfg.get('USER_CONFIG_DEFAULTS', 'apptainerImage')},
     'payload'                     : {'mandatory': True,  'type': str,                        'default': ""},
+    'doCheckpointing'             : {'mandatory': False, 'type': bool,                       'default': False},
+    'checkpointSignal'            : {'mandatory': False, 'type': str,                        'default': ''},
+    'checkpointSecsBeforeTimeout' : {'mandatory': False, 'type': int,                        'default': 0},
+    'checkpointCode'              : {'mandatory': False, 'type': str,                        'default': 'echo "Checkpointing code is left up to the user ..."'},
 })
 
 renamedParams = [
+    {'oldName': 'image', 'newName': 'apptainerImage'},
 ]
 
 deprecatedParams = [
@@ -454,22 +462,12 @@ def validateConfigAfterDefaults(config):
             raise ConfigurationException(msg)
     # Make a sanity check of the input sandbox, batch script and logs filenames.
     paramInvalidRegexMsg = paramInvalidValueMsg + "\nIt should contain only the following characters: a-zA-Z0-9${{}}._-"
-    if config.inputSandboxFilename:
-        if not re.match("^[a-zA-Z0-9${}._-]+$", config.inputSandboxFilename):
-            msg = paramInvalidRegexMsg.format(param='inputSandboxFilename', value=config.inputSandboxFilename)
-            raise ConfigurationException(msg)
-    if config.batchScriptsFilename:
-        if not re.match("^[a-zA-Z0-9${}._-]+$", config.batchScriptsFilename):
-            msg = paramInvalidRegexMsg.format(param='batchScriptsFilename', value=config.batchScriptsFilename)
-            raise ConfigurationException(msg)
-    if config.stdoutFilename:
-        if not re.match("^[a-zA-Z0-9${}._-]+$", config.stdoutFilename):
-            msg = paramInvalidRegexMsg.format(param='stdoutFilename', value=config.stdoutFilename)
-            raise ConfigurationException(msg)
-    if config.stderrFilename:
-        if not re.match("^[a-zA-Z0-9${}._-]+$", config.stderrFilename):
-            msg = paramInvalidRegexMsg.format(param='stderrFilename', value=config.stderrFilename)
-            raise ConfigurationException(msg)
+    cfgParamsToCheck = ['batchScriptsFilename', 'stdoutFilename', 'stderrFilename', 'inputSandboxFilename', 'inputParamsFilename']
+    for paramName in cfgParamsToCheck:
+        if getattr(config, paramName, ''):
+            if not re.match("^[a-zA-Z0-9${}._-]+$", getattr(config, paramName)):
+                msg = paramInvalidRegexMsg.format(param=paramName, value=getattr(config, paramName))
+                raise ConfigurationException(msg)
     # If 'stageoutLogs' is True, then 'writeLogsOnWN' must also be True.
     if config.stageoutLogs and not config.writeLogsOnWN:
         msg = invalidConfigMsg
@@ -498,6 +496,11 @@ def validateConfigAfterDefaults(config):
                 msg = "ERROR: Invalid Configuration:"
                 msg += "\nParameter 'checkpointSecsBeforeTimeout' is set (greater than 0), but parameter 'checkpointSignal' was not specified."
                 raise ConfigurationException(msg)
+    # If 'apptainer' is True, then 'apptainerImage' must be specified.
+    if config.apptainer and not config.apptainerImage:
+        msg = "ERROR: Invalid Configuration:"
+        msg += "\nParameter 'apptainer' is True, but parameter 'apptainerImage' was not specified."
+        raise ConfigurationException(msg)
 
 
 def fixConfigDirs(config):
