@@ -170,39 +170,44 @@ class SubmitWorker:
         # If some parameters need to be adjusted, it should not be done on the config object, but on these variables.
         sbatch_partition            = self.config.sbatch_partition
         sbatch_qos                  = self.config.sbatch_qos
-        sbatch_chdir                = self.config.sbatch_chdir
         sbatch_time                 = self.config.sbatch_time
         sbatch_memPerCPU            = self.config.sbatch_memPerCPU
+        sbatch_chdir                = self.config.sbatch_chdir
         sbatch_output               = self.config.sbatch_output
         sbatch_error                = self.config.sbatch_error
         sbatch_additionalOptions    = self.config.sbatch_additionalOptions
-        writeLogsOnWN               = self.config.writeLogsOnWN
-        stdoutFilename              = self.config.stdoutFilename
-        stderrFilename              = self.config.stderrFilename
-        separateStdoutStderrLogs    = self.config.separateStdoutStderrLogs
-        scratchDir                  = self.config.scratchDir
-        handleScratch               = self.config.handleScratch
-        environmentType             = self.config.environmentType
-        cmsswDir                    = self.config.cmsswDir
+        batchScriptsDir             = self.config.batchScriptsDir
+        batchScriptsFilename        = self.config.batchScriptsFilename
         inputSandboxContent         = self.config.inputSandboxContent
         inputSandboxDir             = self.config.inputSandboxDir
         inputSandboxFilename        = self.config.inputSandboxFilename
-        batchScriptsDir             = self.config.batchScriptsDir
-        batchScriptsFilename        = self.config.batchScriptsFilename
-        stageout                    = self.config.stageout
-        stageoutDir                 = self.config.stageoutDir
-        stageoutFiles               = self.config.stageoutFiles
-        stageoutLogs                = self.config.stageoutLogs
-        stageoutLogsDir             = self.config.stageoutLogsDir
+        scratchDir                  = self.config.scratchDir
+        handleScratch               = self.config.handleScratch
+        writeLogsOnWN               = self.config.writeLogsOnWN
+        separateStdoutStderrLogs    = self.config.separateStdoutStderrLogs
+        stdoutFilename              = self.config.stdoutFilename
+        stderrFilename              = self.config.stderrFilename
+        environmentType             = self.config.environmentType
+        cmsswDir                    = self.config.cmsswDir
         useJobArray                 = self.config.useJobArray
         maxRunningJobs              = self.config.maxRunningJobs
         numJobs                     = self.config.numJobs
+        stageout                    = self.config.stageout
+        stageoutFiles               = self.config.stageoutFiles
+        stageoutDir                 = self.config.stageoutDir
+        stageoutLogs                = self.config.stageoutLogs
+        stageoutLogsDir             = self.config.stageoutLogsDir
         inputParamsNames            = self.config.inputParamsNames
         inputParams                 = self.config.inputParams
+        putInputParamsInBatchScript = self.config.putInputParamsInBatchScript
+        inputParamsFilename         = self.config.inputParamsFilename
+        apptainer                   = self.config.apptainer
+        apptainerImage              = self.config.apptainerImage
+        payload                     = self.config.payload
         doCheckpointing             = self.config.doCheckpointing
         checkpointSignal            = self.config.checkpointSignal
         checkpointSecsBeforeTimeout = self.config.checkpointSecsBeforeTimeout
-        payload                     = self.config.payload
+        checkpointCode              = self.config.checkpointCode
 
         if writeLogsOnWN:
             sbatch_output = '/dev/null'
@@ -301,6 +306,49 @@ class SubmitWorker:
 
         stageoutFiles = [s.strip() for s in stageoutFiles if s.strip()]
 
+        #================================================================================
+        # Prepare the bash code that will set the job input parameters
+        #================================================================================
+
+        # A few lines of bash code to put the job input parameters in environment variables.
+        # This code will be inserted before the payload.
+        # For independent jobs we have one set of lines of code per job.
+        setInputParamsCode = []
+        if numJobInputParams > 0:
+            if self.debug:
+                print("Preparing bash code for reading input parameters for job's payload.")
+            inputParamsStr = ""
+            for j in range(numJobs):
+                inputParamsStr += "\n\"{}\"".format(' <sep> '.join(list(map(str, inputParams[j]))))
+                if j == numJobs - 1: inputParamsStr += "\n"
+            inputParamsStr = "inputParams=({})".format(inputParamsStr)
+            if not putInputParamsInBatchScript:
+                with open(inputParamsFilename, 'w') as fd:
+                    fd.write(inputParamsStr)
+                inputSandboxContent.append(inputParamsFilename)
+                setInputParamsCodeStrBase = "source {}".format(inputParamsFilename)
+            else:
+                setInputParamsCodeStrBase = inputParamsStr
+            for j in range(numJobs):
+                setInputParamsCodeStr = setInputParamsCodeStrBase
+                if useJobArray:
+                    setInputParamsCodeStr += "\nijob=$((${SLURM_ARRAY_TASK_ID}-1))"
+                else:
+                    setInputParamsCodeStr += "\nijob={}".format(j)
+                setInputParamsCodeStr += "\njobInputParams=\"${inputParams[${ijob}]}\""
+                for i in range(numJobInputParams):
+                    setInputParamsCodeStr += "\nexport {paramName}=`echo -e \"${{jobInputParams}}\" | awk -F' <sep> ' '{{print ${paramNum}}}'`".format(paramName=inputParamsNames[i], paramNum=i+1)
+                    setInputParamsCodeStr += "\necho \"{paramName} = ${{{paramName}}}\"".format(paramName=inputParamsNames[i])
+                setInputParamsCode.append(setInputParamsCodeStr)
+                if useJobArray:
+                    break
+            if self.debug:
+                print("Code for reading input parameters for job's payload created.")
+        else:
+            setInputParamsCodeStr = "echo \"No input parameters were defined for the payload.\""
+            if self.debug:
+                print("No code for reading input parameters for job's payload was created.")
+
         #================================================================================
         # Prepare the input sandbox
         #================================================================================
@@ -333,48 +381,12 @@ class SubmitWorker:
                 raise CP3SlurmUtilsException(msg)
             if self.debug:
                 print("Input sandbox created.")
+            if not putInputParamsInBatchScript:
+                subprocess.call(['rm', '-f', inputParamsFilename])
         else:
             if self.debug:
                 print("No input sanbox was created.")
 
-        #================================================================================
-        # Prepare the bash code that will set the job input parameters
-        #================================================================================
-
-        # A few lines of bash code to put the job input parameters in environment variables.
-        # This code will be inserted before the payload.
-        # For independent jobs we have one set of lines of code per job.
-        setInputParamsCode = []
-        if numJobInputParams > 0:
-            if self.debug:
-                print("Preparing bash code for reading input parameters for job's payload.")
-            if useJobArray:
-                inputParamsStr = ""
-                for j in range(numJobs):
-                    inputParamsStr += "\n\"{}\"".format(' <sep> '.join(list(map(str, inputParams[j]))))
-                    if j == numJobs - 1: inputParamsStr += "\n"
-                setInputParamsCodeStr = "inputParams=({})".format(inputParamsStr)
-                setInputParamsCodeStr += "\njobInputParams=\"${inputParams[${SLURM_ARRAY_TASK_ID}-1]}\""
-                for i in range(numJobInputParams):
-                    setInputParamsCodeStr += "\nexport {paramName}=`echo -e \"${{jobInputParams}}\" | awk -F' <sep> ' '{{print ${paramNum}}}'`".format(paramName=inputParamsNames[i], paramNum=i+1)
-                    setInputParamsCodeStr += "\necho \"{paramName} = ${{{paramName}}}\"".format(paramName=inputParamsNames[i])
-                setInputParamsCode.append(setInputParamsCodeStr)
-            else:
-                for j in range(numJobs):
-                    setInputParamsCodeStr = ""
-                    for i in range(numJobInputParams):
-                        if i > 0:
-                            setInputParamsCodeStr += "\n"
-                        setInputParamsCodeStr += "export {paramName}={paramValue}".format(paramName=inputParamsNames[i], paramValue=inputParams[j][i])
-                        setInputParamsCodeStr += "\necho \"{paramName} = ${{{paramName}}}\"".format(paramName=inputParamsNames[i])
-                    setInputParamsCode.append(setInputParamsCodeStr)
-            if self.debug:
-                print("Code for reading input parameters for job's payload created.")
-        else:
-            setInputParamsCodeStr = "echo \"No input parameters were defined for the payload.\""
-            if self.debug:
-                print("No code for reading input parameters for job's payload was created.")
-
         #================================================================================
         # Construct the batch script template
         #================================================================================
@@ -570,7 +582,6 @@ fi
 #==================================================================================
 
 inputSandbox="__INPUT_SANDBOX__"
-inputSandboxDir="$(dirname ${inputSandbox})"
 
 function copy_and_unpack_input_sandbox() {
     case "${inputSandbox}" in
@@ -596,7 +607,7 @@ function copy_and_unpack_input_sandbox() {
     tar -ztvf "${inputSandbox}" | awk '{$1=$2=$3=$4=$5=""; print $0}'
     if [[ ${SLURM_JOB_NUM_NODES} -gt 1 ]]
     then
-        execFile="${inputSandboxDir}/stagein-ISB"
+        execFile="$(dirname ${inputSandbox})/stagein-ISB"
         echo "Will create executable file ${execFile} to be run via srun in order to stagein the input sandbox to all allocated worker nodes." 
         if [[ -f "${execFile}" ]]
         then
@@ -706,7 +717,7 @@ fi
 """\
 
 function checkpoint() {
-    echo "Checkpointing is left up to the user ..."
+__CHECKPOINT_CODE__
 }
 
 function kill_payload() {
@@ -745,9 +756,25 @@ if [[ ${exitCode} -eq 0 ]]
 then
     echo "==== Starting execution of payload (`date`) ===="
     echo "------------------------- Begin payload output ------------------------"
+"""
+        if apptainer:
+            batchScriptTemplate += \
+"""\
+    export APPTAINERENV_PATH=$PATH
+    export APPTAINERENV_LD_LIBRARY_PATH=$LD_LIBRARY_PATH
+    apptainer exec __APPTAINER_IMAGE__ /bin/sh <<- EOF_PAYLOAD &
+__PAYLOAD__
+EOF_PAYLOAD
+"""
+        else:
+            batchScriptTemplate += \
+"""\
     (
 __PAYLOAD__
     ) &
+"""
+        batchScriptTemplate += \
+"""\
     payloadpid=$!
     wait ${payloadpid}
     pec=${pec:-$?}
@@ -1026,29 +1053,31 @@ exit ${exitCode}
         replaceDict = {
             '__SBATCH_PARTITION__': sbatch_partition,
             '__SBATCH_QOS__': sbatch_qos,
-            '__SBATCH_CHDIR__': sbatch_chdir,
             '__SBATCH_TIME__': sbatch_time,
             '__SBATCH_MEM_PER_CPU__': sbatch_memPerCPU,
+            '__SBATCH_CHDIR__': sbatch_chdir,
             '__SBATCH_ARRAY__': jobArrayTaskIds,
             '__SBATCH_OUTPUT__': sbatch_output,
             '__SBATCH_ERROR__': sbatch_error,
             '__SBATCH_ADDITIONAL_OPTIONS__': sbatchAdditionalOptionsStr,
+            '__INPUT_SANDBOX__': inputSandbox,
             '__WRITE_LOGS_ON_WN__': str(writeLogsOnWN).lower(),
             '__SEPARATE_STDOUT_STDERR_LOGS__': str(separateStdoutStderrLogs).lower(),
             '__STDOUT_FILENAME__': stdoutFilename,
             '__STDERR_FILENAME__': stderrFilename,
             '__SCRATCH_DIR__': scratchDir,
             '__CMSSW_DIR__': cmsswDir,
-            '__INPUT_SANDBOX__': inputSandbox,
-            '__PAYLOAD__': payload.strip('\n'),
             '__STAGEOUT__': str(stageout).lower(),
-            '__STAGEOUT_DIR__': stageoutDir,
             '__STAGEOUT_FILES__': '" "'.join(stageoutFiles),
+            '__STAGEOUT_DIR__': stageoutDir,
             '__STAGEOUT_LOGS__': str(stageoutLogs).lower(),
             '__STAGEOUT_LOGS_DIR__': stageoutLogsDir,
-            '__CHECKPOINT_SECS_BEFORE_TIMEOUT__': str(checkpointSecsBeforeTimeout),
+            '__APPTAINER_IMAGE__': apptainerImage,
+            '__PAYLOAD__': payload.strip('\n'),
             '__CHECKPOINT_SIGNAL__': checkpointSignal,
             '__CHECKPOINT_SIGNAL_SHORT__': checkpointSignal.replace('SIG', ''),
+            '__CHECKPOINT_SECS_BEFORE_TIMEOUT__': str(checkpointSecsBeforeTimeout),
+            '__CHECKPOINT_CODE__': checkpointCode.strip('\n'),
 
         }
 
@@ -1162,7 +1191,7 @@ exit ${exitCode}
                 if not self.quiet:
                     print("Please read first the summary given above, then answer whether you really want to submit the jobs.")
                     print("N.B.: You can always submit the jobs (i.e. batch scripts) yourself with the slurm 'sbatch' command.")
-                question = "Are you sure you want to submit the jobs ? (you have {} minutes to answer)".format(timeout)
+                question = "Are you sure you want to submit the jobs? (you have {} minutes to answer)".format(timeout)
                 signal.signal(signal.SIGALRM, questionTimeoutHandler)
                 signal.alarm(timeout*60)
                 try:
